@@ -11,8 +11,8 @@ GTRIG_FUNCTION  = 1 # unsupported
 GTRIG_COMMAND   = 2
 GTRIG_MEDIAPLAY = 3
 
-def readfile(filename):
-	_gnome.gnome_triggers_readfile(filename)
+#def readfile(filename):
+#	_gnome.gnome_triggers_readfile(filename)
 def add_trigger(trig_type, trig_arg, trig_level, *supinfo):
 	_gnome.gnome_triggers_add_trigger(trig_type, trig_arg, trig_level,
 					  supinfo)
