@@ -19,11 +19,5 @@ class GtkHTML (_gtk.GtkLayout):
     def end (self, handle, status):
         _gtkhtml.gtk_html_end (self._o, handle, status)
 
-    def parse (self):
-        _gtkhtml.gtk_html_parse (self._o)
-
-    def calc_scrollbars (self):
-        _gtkhtml.gtk_calc_scrollbars (self._o)
-
 _gtk._name2cls['GtkHTML'] = GtkHTML
 
