@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <Python.h>
 #include <gtk-xmhtml/gtk-xmhtml.h>
 
