@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset: 4 -*- */
 /* PyGTK imlib module - python bindings for gdk_imlib
  * Copyright (C) 1998 James Henstridge <james@daa.com.au>
  *
@@ -17,95 +18,87 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <Python.h>
 #include <gtk/gtk.h>
 #include <gdk_imlib.h>
 
+#ifdef HAVE_NUMPY
+#  if defined(HAVE_ARRAYOBJECT_H)
+#    include <arrayobject.h>
+#  elif defined(HAVE_EXTESNSIONS_ARRAYOBJECT_H)
+#    include <Extensions/arrayobject.h>
+#  elif defined(HAVE_NUMERIC_ARRAYOBJECT_H)
+#    include <Numeric/arrayobject.h>
+#  else
+#    error "arrayobject.h not found, but HAVE_NUMPY defined"
+#  endif
+#endif
 
-typedef struct {
-    PyObject_HEAD
-    GtkObject *go;
-} PyGtk_Object;
-PyTypeObject *PyGtk_Type;
-#define PyGtk_Check(op) ((op)->ob_type == PyGtk_Type)
+#include "pygtk.h"
 
-static PyObject *(*PyGtk_New)(GtkObject *go);
-
-typedef struct {
-    PyObject_HEAD
-    GdkWindow *obj;
-} PyGdkWindow_Object;
-PyTypeObject *PyGdkWindow_Type;
-#define PyGdkWindow_Check(op) ((op)->ob_type == PyGdkWindow_Type)
-#define PyGdkWindow_Get(op) (((PyGdkWindow_Object *)(op))->obj)
-static PyObject *(*PyGdkWindow_New)(GdkWindow *win);
-
-
-typedef struct {
-	PyObject_HEAD
-	GdkImlibImage *obj;
-} PyGdkImlibImage_Object;
-
-staticforward PyTypeObject PyGdkImlibImage_Type;
-#define PyGdkImlibImage_Check(ob) ((ob)->ob_type == &PyGdkImlibImage_Type)
-#define PyGdkImlibImage_Get(ob) (((PyGdkImlibImage_Object *)(ob))->obj)
+#define _INSIDE_PYGDKIMLIB_
+#include "pygdkimlib.h"
 
 static PyObject *
 PyGdkImlibImage_New(GdkImlibImage *obj) {
-  PyGdkImlibImage_Object *self;
+    PyGdkImlibImage_Object *self;
 
-  if (obj == NULL) {
-      PyErr_SetString(PyExc_IOError, "couldn't load image");
-      return NULL;
-  }
+    if (obj == NULL) {
+	PyErr_SetString(PyExc_IOError, "couldn't load image");
+	return NULL;
+    }
 
-  self = (PyGdkImlibImage_Object *)PyObject_NEW(PyGdkImlibImage_Object,
-						&PyGdkImlibImage_Type);
-  if (self == NULL)
-    return NULL;
-  self->obj = obj;
-  return (PyObject *)self;
+    self = (PyGdkImlibImage_Object *)PyObject_NEW(PyGdkImlibImage_Object,
+						  &PyGdkImlibImage_Type);
+    if (self == NULL)
+	return NULL;
+    self->obj = obj;
+    return (PyObject *)self;
 }
 
 static void
 PyGdkImlibImage_Dealloc(PyGdkImlibImage_Object *self) {
-  gdk_imlib_destroy_image(self->obj); 
-  PyMem_DEL(self);
+    gdk_imlib_destroy_image(self->obj); 
+    PyMem_DEL(self);
 }
 
 static int
 PyGdkImlibImage_Compare(PyGdkImlibImage_Object *self,
 			PyGdkImlibImage_Object *v) {
-  if (self->obj == v->obj) return 0;
-  if (self->obj > v->obj) return -1;
-  return 1;
+    if (self->obj == v->obj) return 0;
+    if (self->obj > v->obj) return -1;
+    return 1;
 }
 
 static PyTypeObject PyGdkImlibImage_Type = {
-  PyObject_HEAD_INIT(&PyType_Type)
-  0,
-  "GdkImlibImage",
-  sizeof(PyGdkImlibImage_Object),
-  0,
-  (destructor)PyGdkImlibImage_Dealloc,
-  (printfunc)0,
-  (getattrfunc)0,
-  (setattrfunc)0,
-  (cmpfunc)PyGdkImlibImage_Compare,
-  (reprfunc)0,
-  0,
-  0,
-  0,
-  (hashfunc)0,
-  (ternaryfunc)0,
-  (reprfunc)0,
-  0L,0L,0L,0L,
-  NULL
+    PyObject_HEAD_INIT(&PyType_Type)
+    0,
+    "GdkImlibImage",
+    sizeof(PyGdkImlibImage_Object),
+    0,
+    (destructor)PyGdkImlibImage_Dealloc,
+    (printfunc)0,
+    (getattrfunc)0,
+    (setattrfunc)0,
+    (cmpfunc)PyGdkImlibImage_Compare,
+    (reprfunc)0,
+    0,
+    0,
+    0,
+    (hashfunc)0,
+    (ternaryfunc)0,
+    (reprfunc)0,
+    0L,0L,0L,0L,
+    NULL
 };
 
 
 static PyObject *_wrap_gdk_imlib_init(PyObject *self, PyObject *args) {
-    static called = 0;
+    static int called = 0;
 
     if (called) {
         Py_INCREF(Py_None);
@@ -155,11 +148,11 @@ static PyObject *_wrap_gdk_imlib_load_image(PyObject *self, PyObject *args) {
 }
 
 static PyObject *_wrap_gdk_imlib_best_color_match(PyObject *self, PyObject *args) {
-  int r, g, b;
-  if (!PyArg_ParseTuple(args, "iii:gdk_imlib_best_color_match", &r, &g, &b))
-    return NULL;
-  gdk_imlib_best_color_match(&r, &g, &b);
-  return Py_BuildValue("(iii)", r, g, b);
+    int r, g, b;
+    if (!PyArg_ParseTuple(args, "iii:gdk_imlib_best_color_match", &r, &g, &b))
+	return NULL;
+    gdk_imlib_best_color_match(&r, &g, &b);
+    return Py_BuildValue("(iii)", r, g, b);
 }
 
 static PyObject *_wrap_gdk_imlib_render(PyObject *self, PyObject *args) {
@@ -228,7 +221,7 @@ static PyObject *_wrap_gdk_imlib_free_colors(PyObject *self, PyObject *args) {
 
 static PyObject *_wrap_gdk_imlib_get_image_border(PyObject *self, PyObject *args) {
     GdkImlibBorder border;
-    PyObject *image ;
+    PyObject *image;
 
     if(!PyArg_ParseTuple(args,"O!:gdk_imlib_get_image_border",
 			 &PyGdkImlibImage_Type, &image)) 
@@ -419,6 +412,7 @@ static PyObject *_wrap_gdk_imlib_set_image_red_curve(PyObject *self, PyObject *a
     }
     for (i = 0; i < 256; i++) {
         item = PySequence_GetItem(list, i);
+        Py_DECREF(item);
         if (!PyInt_Check(item)) {
 	    PyErr_SetString(PyExc_TypeError,"item of sequence not an integer");
 	    return NULL;
@@ -444,6 +438,7 @@ static PyObject *_wrap_gdk_imlib_set_image_green_curve(PyObject *self, PyObject
     }
     for (i = 0; i < 256; i++) {
         item = PySequence_GetItem(list, i);
+        Py_DECREF(item);
         if (!PyInt_Check(item)) {
 	    PyErr_SetString(PyExc_TypeError,"item of sequence not an integer");
 	    return NULL;
@@ -469,6 +464,7 @@ static PyObject *_wrap_gdk_imlib_set_image_blue_curve(PyObject *self, PyObject *
     }
     for (i = 0; i < 256; i++) {
         item = PySequence_GetItem(list, i);
+        Py_DECREF(item);
         if (!PyInt_Check(item)) {
 	    PyErr_SetString(PyExc_TypeError,"item of sequence not an integer");
 	    return NULL;
@@ -601,6 +597,29 @@ static PyObject *_wrap_gdk_imlib_create_image_from_data(PyObject *self, PyObject
 					data, alpha, w, h));
 }
 
+static PyObject *_wrap_gdk_imlib_create_image_from_xpm(PyObject *self, PyObject *args) {
+    PyObject *lines;
+    int len, i;
+    char **data;
+
+    if (!PyArg_ParseTuple(args, "O!:gdk_imlib_create_image_from_data",
+			  &PyList_Type, &lines))
+	return NULL;
+
+    len = PyList_Size(lines);
+    data = g_new(char *, len);
+    for (i = 0; i < len; i++) {
+	PyObject *item = PyList_GetItem(lines, i);
+	if (!PyString_Check(item)) {
+	    PyErr_SetString(PyExc_TypeError, "list items must be strings");
+	    g_free(data);
+	    return NULL;
+	}
+	data[i] = PyString_AsString(item);
+    }
+    return PyGdkImlibImage_New(gdk_imlib_create_image_from_xpm_data(data));
+}
+
 static PyObject *_wrap_gdk_imlib_clone_image(PyObject *self, PyObject *args) {
     PyObject *image;
 
@@ -741,6 +760,67 @@ static PyObject *_wrap_gdk_imlib_pop_visual(PyObject *self, PyObject *args) {
     return Py_None;
 }
 
+#ifdef HAVE_NUMPY
+static PyObject *_wrap_gdk_imlib_create_image_from_array(PyObject *self, PyObject *args) {
+    PyArrayObject *ap_data, *ap_alpha=NULL;
+    unsigned char *data, *alpha = NULL;
+    int w, h;
+    if (!PyArg_ParseTuple(args, "O!|O!:gdk_imlib_create_image_from_array",
+			  &PyArray_Type, &ap_data, &PyArray_Type, &ap_alpha))
+	return NULL;
+    if (!PyArray_ISCONTIGUOUS(ap_data) ||
+	ap_data->descr->type_num != PyArray_UBYTE ||
+	ap_data->nd != 3 ||
+	ap_data->dimensions[2] != 3) {
+	PyErr_SetString(PyExc_TypeError,
+		"image array must be MxNx3 contiguous unsigned byte array");
+	return NULL;
+    }
+    data = (unsigned char *)ap_data->data;
+    h = ap_data->dimensions[0];
+    w = ap_data->dimensions[1];
+
+    if (ap_data != NULL && (!PyArray_ISCONTIGUOUS(ap_alpha) ||
+			    ap_alpha->descr->type_num != PyArray_UBYTE ||
+			    ap_alpha->nd != 2 ||
+			    ap_alpha->dimensions[0] != h ||
+			    ap_alpha->dimensions[1] != w)) {
+	PyErr_SetString(PyExc_TypeError,
+		"alpha channel array must be 2D contiguous byte array");
+	return NULL;
+    }
+    if (ap_alpha != NULL)
+	alpha = (unsigned char *)ap_alpha->data;
+    return PyGdkImlibImage_New(gdk_imlib_create_image_from_data(
+						data, alpha, w, h));
+}
+
+static PyObject *_wrap_gdk_imlib_image_get_array(PyObject *self, PyObject *args) {
+    PyObject *image;
+    GdkImlibImage *obj;
+    PyArrayObject *ap_data;
+    int dims[3] = {0, 0, 3};
+
+    if (!PyArg_ParseTuple(args, "O!:gdk_imlib_image_get_array",
+			  &PyGdkImlibImage_Type, &image))
+	return NULL;
+    obj = PyGdkImlibImage_Get(image);
+    dims[0] = (int)obj->rgb_width;
+    dims[1] = (int)obj->rgb_height;
+
+    ap_data = (PyArrayObject *)PyArray_FromDimsAndData(3, dims, PyArray_UBYTE,
+						       (char *)obj->rgb_data);
+    if (ap_data == NULL)
+	return NULL;
+    /* this makes sure the GdkImlibImage is not freed while someone has a
+     * reference to the new array. */
+    Py_INCREF(image);
+    ap_data->base = image;
+
+    return PyArray_Return(ap_data);
+}
+#endif
+
 static PyMethodDef _gdkimlibMethods[] = {
     { "gdk_imlib_push_visual", _wrap_gdk_imlib_push_visual, 1 },
     { "gdk_imlib_pop_visual", _wrap_gdk_imlib_pop_visual, 1 },
@@ -757,6 +837,7 @@ static PyMethodDef _gdkimlibMethods[] = {
     { "gdk_imlib_clone_scaled_image", _wrap_gdk_imlib_clone_scaled_image, 1 },
     { "gdk_imlib_clone_image", _wrap_gdk_imlib_clone_image, 1 },
     { "gdk_imlib_create_image_from_data", _wrap_gdk_imlib_create_image_from_data, 1 },
+    { "gdk_imlib_create_image_from_xpm", _wrap_gdk_imlib_create_image_from_xpm, 1 },
     { "gdk_imlib_rotate_image", _wrap_gdk_imlib_rotate_image, 1 },
     { "gdk_imlib_flip_image_vertical", _wrap_gdk_imlib_flip_image_vertical, 1 },
     { "gdk_imlib_flip_image_horizontal", _wrap_gdk_imlib_flip_image_horizontal, 1 },
@@ -793,18 +874,33 @@ static PyMethodDef _gdkimlibMethods[] = {
     { "gdk_imlib_set_render_type", _wrap_gdk_imlib_set_render_type, 1 },
     { "gdk_imlib_get_render_type", _wrap_gdk_imlib_get_render_type, 1 },
     { "gdk_imlib_init", _wrap_gdk_imlib_init, 1 },
+#ifdef HAVE_NUMPY
+    { "gdk_imlib_create_image_from_array", _wrap_gdk_imlib_create_image_from_data, 1 },
+    { "gdk_imlib_image_get_array", _wrap_gdk_imlib_image_get_array, 1 },
+#endif
     { NULL, NULL }
 };
 
+static struct _PyGdkImlib_FunctionStruct functions = {
+    &PyGdkImlibImage_Type, PyGdkImlibImage_New
+};
+
 void init_gdkimlib() {
     PyObject *m, *d, *private;
 
     m = Py_InitModule("_gdkimlib", _gdkimlibMethods);
     d = PyModule_GetDict(m);
+
+#ifdef HAVE_NUMPY
+    import_array();
+#endif
     
     PyDict_SetItemString(d, "GdkImlibImageType",
 			 (PyObject *)&PyGdkImlibImage_Type);
 
+     PyDict_SetItemString(d, "_PyGdkI_API",
+			  PyCObject_FromVoidPtr(&functions, NULL));
+
     private = PyDict_New();
     PyDict_SetItemString(d, "_private", private); Py_DECREF(private);
 
@@ -812,25 +908,8 @@ void init_gdkimlib() {
 			 d=PyCObject_FromVoidPtr(PyGdkImlibImage_New, NULL));
     Py_DECREF(d);
 
-    /* get types and exported functions out of _gtkmodule */
-    m = PyImport_ImportModule("_gtk");
-    if (m == NULL) {
-        Py_FatalError("couldn't import _gtk");
-	return;
-    }
-    d = PyModule_GetDict(m);
-    Py_DECREF(m);
-
-    PyGtk_Type = (PyTypeObject *)PyDict_GetItemString(d, "GtkObjectType");
-    Py_INCREF(PyGtk_Type);
-    PyGdkWindow_Type = (PyTypeObject *)PyDict_GetItemString(d,"GdkWindowType");
-    Py_INCREF(PyGdkWindow_Type);
-
-    d = PyDict_GetItemString(d, "_private");
-    PyGtk_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d, "PyGtk_New"));
-    PyGdkWindow_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-							"PyGdkWindow_New"));
-
+    /* get references to the pygtk types and functions */
+    init_pygtk();
 
     if (PyErr_Occurred())
          Py_FatalError("can't initialise module _gdkimlib");
